/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2019                               **
**                                                                        **
**                             MODULE trace                               **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2019.01.19                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#include "trace.h"
#include "uart0.h"
#include "strings.h"
#include "frameutil.h"


#define traceDHCP_BootRequest               1
#define traceDHCP_BootReply                 2
#define DHCP_PadCode                   0
#define DHCP_SubNetMaskCode            1
#define DHCP_RouterCode                3
#define DHCP_HostNameCode              12
#define DHCP_RequestedIPCode           50
#define DHCP_LeaseTimeCode             51
#define DHCP_MessageTypeCode           53
#define DHCP_ServerIDCode              54
#define DHCP_ParameterRequestListeCode 55
#define DHCP_ClientCode                61
#define DHCP_EndCode                   255

/*  DHCP message types.  */
#define DHCP_DHCPDISCOVER              1
#define DHCP_DHCPOFFER                 2
#define DHCP_DHCPREQUEST               3
#define DHCP_DHCPDECLINE               4
#define DHCP_DHCPACK                   5
#define DHCP_DHCPNACK                  6
#define DHCP_DHCPRELEASE               7
#define DHCP_DHCPINFORM                8
static ULONG GetUL ( UCHAR * DataBuffer )
{
  ULONG Temp ;
  /*-------------------------------------------------------------------------*/
  Temp = ( ULONG ) ( * DataBuffer ++ ) ;
  Temp = ( Temp << 8 ) | ( ULONG ) ( * DataBuffer ++ ) ;
  Temp = ( Temp << 8 ) | ( ULONG ) ( * DataBuffer ++ ) ;
  Temp = ( Temp << 8 ) | ( ULONG ) ( * DataBuffer ++ ) ;
  return ( Temp ) ;
} /* GetUL */

static UCHAR Message60 [ ]          = "\r\nPakiet DHCP:\r\nopt=" ;
static UCHAR Message61 [ ]          = " zapytanie/zadanie" ;
static UCHAR Message62 [ ]          = " odpowiedz od serwera DHCP" ;
static UCHAR Message63 [ ]          = "???" ;
static UCHAR Message64 [ ]          = "\r\nHTYPE=" ;
static UCHAR Message65 [ ]          = "\r\nHLEN=" ;
static UCHAR Message66 [ ]          = "\r\nXID=" ;
static UCHAR Message67 [ ]          = "\r\nciadr=" ;
static UCHAR Message68 [ ]          = "\r\nyiadr=" ;
static UCHAR Message69 [ ]          = "\r\nsiadr=" ;
static UCHAR Message70 [ ]          = "\r\ngiadr=" ;
static UCHAR Message71 [ ]          = "\r\nchadr=" ;
static UCHAR Message72 [ ]          = "\r\nsname=" ;
static UCHAR Message73 [ ]          = "\r\nmagic cookie=" ;
static UCHAR Message74 [ ]          = "\r\nmsg type=" ;
static UCHAR Message75 [ ]          = "\r\nserwer ID=" ;
static UCHAR Message76 [ ]          = "\r\nmaska podsieci=" ;
static UCHAR Message77 [ ]          = "\r\nbrama domyslna=" ;
static UCHAR Message78 [ ]          = "\r\nczas dzierzawy=" ;
static UCHAR Message79 [ ]          = "\r\npole opcji=??? =" ;
static UCHAR Message80 [ ]          = " faza DHCP_DHCPOFFER" ;
static UCHAR Message81 [ ]          = " faza DHCP_DHCPACK ------> koniec negocjacji" ;
static UCHAR Message82 [ ]          = " faza DHCP_DHCPNACK -> nie przydzielono, restart negocjacji" ;
static UCHAR Message83 [ ]          = " faza DHCP_DHCPDISCOVER" ;
static UCHAR Message84 [ ]          = " faza DHCP_DHCPREQUEST" ;
static UCHAR Message85 [ ]          = "\r\nkoniec ciasteczek" ;
static UCHAR Message86 [ ]          = "\r\nidentyfikator klienta=" ;
static UCHAR Message87 [ ]          = "\r\nzadany adres IP=" ;

void traceDHCP ( UCHAR * UDPDataBuffer , USHORT UDPDataSize )
{
  UCHAR * ChPtr ;
  UCHAR Ch ;
  ULONG UL;
  USHORT OptionSize ;
//  USHORT SourcePortNo ;
//  USHORT DestinationPortNo ;
//  USHORT UDPPackedSize ;
//  USHORT UDPCheckSum ;
  /*-------------------------------------------------------------------------*/
  ChPtr = UDPDataBuffer ;
  UART0SendString ( Message60 ) ;
  Ch = * ChPtr ++ ; UDPDataSize -- ;
  TraceUCHAR ( Ch ) ;
  switch ( Ch )
  {
    case traceDHCP_BootRequest               :
      UART0SendString ( Message61 ) ;
      break ;
    case traceDHCP_BootReply                 :
      UART0SendString ( Message62 ) ;
      break ;
    default :
      UART0SendString ( Message63 ) ;
  } /* switch */ ;
  Ch = * ChPtr ++ ; UDPDataSize -- ;
  UART0SendString ( Message64 ) ;
  TraceUCHAR ( Ch ) ;
  Ch = * ChPtr ++ ; UDPDataSize -- ;
  UART0SendString ( Message65 ) ;
  TraceUCHAR ( Ch ) ;
  Ch = * ChPtr ++ ; UDPDataSize -- ;
  UL = GetUL ( ChPtr ) ;
  ChPtr += 8 ; UDPDataSize -=8 ;
  UART0SendString ( Message66 ) ;
  TraceULONG ( UL );
  UL = GetUL ( ChPtr ) ;
  ChPtr += 4 ; UDPDataSize -=4 ;
  UART0SendString ( Message67 ) ;
  TraceIP ( UL );
  UL = GetUL ( ChPtr ) ;
  ChPtr += 4 ; UDPDataSize -=4 ;
  UART0SendString ( Message68 ) ;
  TraceIP ( UL );
  UL = GetUL ( ChPtr ) ;
  ChPtr += 4 ; UDPDataSize -=4 ;
  UART0SendString ( Message69 ) ;
  TraceIP ( UL );
  UL = GetUL ( ChPtr ) ;
  ChPtr += 4 ; UDPDataSize -=4 ;
  UART0SendString ( Message70 ) ;
  TraceIP ( UL );
  UART0SendString ( Message71 ) ;
  TraceMAC ( ChPtr );
  ChPtr += 16 ; UDPDataSize -=16 ;
  UART0SendString ( Message72 ) ;
  UART0SendString ( ChPtr ) ;
  ChPtr += 64 ; UDPDataSize -=64 ;
  ChPtr += 128 ; UDPDataSize -=128 ;
  UL = GetUL ( ChPtr ) ;
  ChPtr += 4 ; UDPDataSize -=4 ;
  UART0SendString ( Message73 ) ;
  TraceULONG ( UL );
  while ( UDPDataSize )
  {
    Ch = * ChPtr ++ ; UDPDataSize -- ;
    if ( Ch == DHCP_PadCode )
    {
      continue ;
    } /* if */ ;
    if ( Ch == DHCP_EndCode )
    {
      UART0SendString ( Message85 ) ;
      break ;
    } /* if */ ;
    OptionSize = * ChPtr ++ ; UDPDataSize -- ;
    if ( OptionSize > UDPDataSize )
    {
      break ;
    } /* if */ ;
    switch ( Ch )
    {
      case DHCP_MessageTypeCode :
        if ( OptionSize == 1 )
        {
          Ch = * ChPtr ++ ; UDPDataSize -- ;
          UART0SendString ( Message74 ) ;
          TraceUCHAR ( Ch ) ;
          switch ( Ch )
          {
            case DHCP_DHCPOFFER :
              UART0SendString ( Message80 ) ;
              break ;
            case DHCP_DHCPACK :
              UART0SendString ( Message81 ) ;
              break ;
            case DHCP_DHCPNACK :
              UART0SendString ( Message82 ) ;
              break ;
            case DHCP_DHCPDISCOVER :
              UART0SendString ( Message83 ) ;
              break ;
            case DHCP_DHCPREQUEST :
              UART0SendString ( Message84 ) ;
              break ;
          } /* switch */ ;
        OptionSize = 0 ;
        } /* if */ ;
        break ;
      case DHCP_ServerIDCode :
        if ( OptionSize == 4 )
        {
          UL = GetUL ( ChPtr ) ;
          ChPtr += 4 ; UDPDataSize -= 4 ;
          UART0SendString ( Message75 ) ;
          TraceIP ( UL );
          OptionSize = 0 ;
        } /* if */ ;
        break ;
      case DHCP_SubNetMaskCode :
        if ( OptionSize == 4 )
        {
          UL = GetUL ( ChPtr ) ;
          ChPtr += 4 ; UDPDataSize -= 4 ;
          UART0SendString ( Message76 ) ;
          TraceIP ( UL );
          OptionSize = 0 ;
        } /* if */ ;
        break ;
      case DHCP_RouterCode :
        if ( OptionSize == 4 )
        {
          UL = GetUL ( ChPtr ) ;
          ChPtr += 4 ; UDPDataSize -= 4 ;
          UART0SendString ( Message77 ) ;
          TraceIP ( UL );
          OptionSize = 0 ;
        } /* if */ ;
        break ;
      case DHCP_LeaseTimeCode :
        if ( OptionSize == 4 )
        {
          UL = GetUL ( ChPtr ) ;
          ChPtr += 4 ; UDPDataSize -= 4 ;
          UART0SendString ( Message78 ) ;
          TraceULONG ( UL );
          OptionSize = 0 ;
        } /* if */ ;
        break ;
      case DHCP_PadCode                   :
        break ;
      case DHCP_RequestedIPCode           :
        UL = GetUL ( ChPtr ) ;
        ChPtr += 4 ; UDPDataSize -= 4 ;
        UART0SendString ( Message87 ) ;
        TraceIP ( UL );
        OptionSize = 0 ;
        break ;
      case DHCP_ParameterRequestListeCode :
        break ;
      case DHCP_HostNameCode                :
        UART0SendString ( Message86 ) ;
        if ( OptionSize )
          UART0SendString ( ChPtr ) ;
        ChPtr += OptionSize ;
        UDPDataSize -= OptionSize ;
        OptionSize = 0 ;
        break ;
      case DHCP_EndCode                   :
        UART0SendString ( Message85 ) ;
        break ;
      default :
        UART0SendString ( Message79 ) ;
        TraceUSHORTdec ( Ch ) ;
    } /* switch */ ;
    if ( OptionSize )
    {
      ChPtr += OptionSize ;
      UDPDataSize -= OptionSize ;
    } /* if */ ;
  } /* while */ ;
  UART0SendString ( (UCHAR*)"\r\n\r\n\r\n" ) ;
} /* traceDHCP */


static UCHAR Message50 [ ]          = "\r\nNaglowek UDP:" ;
static UCHAR Message51 [ ]          = "\r\nSRCPORT=" ;
static UCHAR Message52 [ ]          = " DSTPORT=" ;
static UCHAR Message53 [ ]          = "\r\nLGTH=" ;
static UCHAR Message54 [ ]          = " UDPSUM=" ;

void TraceUDPHeader ( PtrOSFrameRecT Frame )
{
  USHORT SourcePortNo ;
  USHORT DestinationPortNo ;
  USHORT UDPPackedSize ;
  USHORT UDPCheckSum ;
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message50 ) ;
  SourcePortNo = GetUSHORTSwapField ( Frame , 0x22 ) ;
  DestinationPortNo = GetUSHORTSwapField ( Frame , 0x024 ) ;
  UDPPackedSize = GetUSHORTSwapField ( Frame , 0x26 ) ;
  UDPCheckSum = GetUSHORTSwapField ( Frame , 0x28 ) ;
  UART0SendString ( Message51 ) ;
  TraceUSHORTdec ( SourcePortNo ) ;
  UART0SendString ( Message52 ) ;
  TraceUSHORTdec ( DestinationPortNo ) ;
  UART0SendString ( Message53 ) ;
  TraceUSHORTdec ( UDPPackedSize ) ;
  UART0SendString ( Message54 ) ;
  TraceUSHORT ( UDPCheckSum ) ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
} /* TraceUDPHeader */


static UCHAR TMessage01 [ ]      = "Ramka ETH, DEST=" ;
static UCHAR TMessage02 [ ]      = " SRC=" ;
static UCHAR TMessage03 [ ]      = "\r\nPakiet ARP" ;
static UCHAR TMessage04 [ ]      = "\r\nPakiet IP" ;
static UCHAR TMessage05 [ ]      = "\r\nPakiet ???" ;
static UCHAR TMessage06 [ ]      = "/ICMP" ;
static UCHAR TMessage07 [ ]      = "/TCP" ;
static UCHAR TMessage08 [ ]      = "/UDP" ;
static UCHAR TMessage09 [ ]      = "/???" ;

void TraceFrame ( PtrOSFrameRecT Frame )
{
  EtherPackedHeaderRecT EtherHeader ;
  IPHeaderRecT IPHeader ;
  /*-------------------------------------------------------------------------*/
  GetEthHeader ( & EtherHeader , Frame ) ;
  UART0SendString ( TMessage01 ) ;
  TraceMAC ( EtherHeader . DestinMACAddress ) ;
  UART0SendString ( TMessage02 ) ;
  TraceMAC ( EtherHeader . SourceMACAddress ) ;
  switch ( EtherHeader . LengthOrType )
  {
    case FRAME_ARP :
      UART0SendString ( TMessage03 ) ;
      break ;
    case FRAME_IP :
      UART0SendString ( TMessage04 ) ;
      SetIPHeaderOrgin ( Frame ) ; 
      GetIPHeader ( & IPHeader , Frame ) ;
      switch ( IPHeader . Protocol )
      {
        case Prot_ICMP                :
          UART0SendString ( TMessage06 ) ;
          break ;
        case Prot_TCP                 :
          UART0SendString ( TMessage07 ) ;
          break ;
        case Prot_UDP                 :
          UART0SendString ( TMessage08 ) ;
          break ;
        default :
          UART0SendString ( TMessage09 ) ;
       } /* switch */ ;
      break ;
    default :
      UART0SendString ( TMessage05 ) ;
  } /* switch */ ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
} /* TraceFrame */

static UCHAR IPMessage10 [ ]          = "\r\nNaglowek IP:" ;
static UCHAR IPMessage11 [ ]          = "\r\nVERS=" ;
static UCHAR IPMessage12 [ ]          = " HLEN=" ;
static UCHAR IPMessage13 [ ]          = " TOS=" ;
static UCHAR IPMessage14 [ ]          = " TOTAL=" ;
static UCHAR IPMessage15 [ ]          = "\r\nIDENT=" ;
static UCHAR IPMessage16 [ ]          = " FLAGS=" ;
static UCHAR IPMessage17 [ ]          = " FRAGOFFS=" ;
static UCHAR IPMessage18 [ ]          = "\r\nTTL=" ;
static UCHAR IPMessage19 [ ]          = " PROT=" ;
static UCHAR IPMessage20 [ ]          = " HCRC=" ;
static UCHAR IPMessage21 [ ]          = "\r\nSRC=" ;
static UCHAR IPMessage22 [ ]          = "\r\nDST=" ;

void TraceIPHeader ( IPHeaderRecT * IPHeader )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( IPMessage10 ) ;
  UART0SendString ( IPMessage11 ) ;
  TraceUCHAR ( IPHeader -> Vers ) ;
  UART0SendString ( IPMessage12 ) ;
  TraceUCHAR ( IPHeader -> HLen ) ;
  UART0SendString ( IPMessage13 ) ;
  TraceUCHAR ( IPHeader -> ServiceType ) ;
  UART0SendString ( IPMessage14 ) ;
  TraceUSHORT ( IPHeader -> TotalLength ) ;
  UART0SendString ( IPMessage15 ) ;
  TraceUSHORT ( IPHeader -> Identification ) ;
  UART0SendString ( IPMessage16 ) ;
  TraceUCHAR ( IPHeader -> Flags ) ;
  UART0SendString ( IPMessage17 ) ;
  TraceUSHORT ( IPHeader -> FragmentOffs ) ;
  UART0SendString ( IPMessage18 ) ;
  TraceUCHAR ( IPHeader -> TTL ) ;
  UART0SendString ( IPMessage19 ) ;
  TraceUSHORT ( IPHeader -> Protocol ) ;
  UART0SendString ( IPMessage20 ) ;
  TraceUSHORT ( IPHeader -> HeaderChecksum ) ;
  UART0SendString ( IPMessage21 ) ;
  TraceIP ( IPHeader -> SourceIPAddress ) ;
  UART0SendString ( IPMessage22 ) ;
  TraceIP ( IPHeader -> DestinationIPAddress ) ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
} /* TraceIPHeader */


UCHAR Message01 [ ]              = "\r\nOdebrano ramke do MAC=" ;
UCHAR Message02 [ ]              = " od MAC=" ;
UCHAR Message03 [ ]              = ", jest to pakiet ARP\r\n" ;
UCHAR Message04 [ ]              = ", jest to pakiet IP\r\n" ;
UCHAR Message05 [ ]              = ", jest to pakiet ???\r\n" ;

void TraceETHHeader ( EtherPackedHeaderRecT * EtherPackedHeader )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message01 ) ;
  TraceMAC ( EtherPackedHeader -> DestinMACAddress ) ;
  UART0SendString ( Message02 ) ;
  TraceMAC ( EtherPackedHeader -> SourceMACAddress ) ;
  switch ( EtherPackedHeader -> LengthOrType )
  {
    case FRAME_ARP    :
      UART0SendString ( Message03 ) ;
      break ;
    case FRAME_IP     :
      UART0SendString ( Message04 ) ;
      break ;
    default :
      UART0SendString ( Message05 ) ;
  } /* switch */ ;
} /* TraceETHHeader */


void TraceMAC ( UCHAR * MACAddr )
{  
  UCHAR NumberStr [ 32 ] ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 5 ; Loop ++ )
  {
    WordToHexStr ( NumberStr , 2 , '0' ,  ( USHORT ) ( * MACAddr ) ) ;
    MACAddr ++ ;
    UART0SendString ( NumberStr ) ;
    UART0Send ( '-' ) ;
  } /* for */ ;  
  WordToHexStr ( NumberStr , 2 , '0' ,  ( USHORT ) ( * MACAddr ) ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceMAC */


void TraceIP ( ULONG IPAddress )
{
  UCHAR NumberStr [ 24 ] ;
  /*-------------------------------------------------------------------------*/
  IPToString ( NumberStr , IPAddress ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceIP */


void TraceUCHAR ( UCHAR Data )
{
  UCHAR NumberStr [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceUCHAR */
  

void TraceUSHORT ( USHORT Data )
{
  UCHAR NumberStr [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  WordToHexStr ( NumberStr , 4 , '0' , Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceUSHORT */


void TraceUSHORTdec ( USHORT Data )
{
  UCHAR NumberStr [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  WordToStr ( NumberStr , 5 , '0' , Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceUSHORTdec */


void TraceULONG ( ULONG Data )
{
  UCHAR NumberStr [ 12 ] ;
  /*-------------------------------------------------------------------------*/
  LongToHexStr ( NumberStr , 8 , '0' , Data ) ;
  UART0SendString ( NumberStr ) ;
} /* TraceULONG */


void traceBlock ( UCHAR * Data , USHORT Length )
{
  UCHAR NumberStr [ 8 ] ;
  USHORT Cnt ;
  /*-------------------------------------------------------------------------*/
  UART0SendString ( ( UCHAR * ) "Dlugosc=" ) ;
  TraceUSHORTdec ( Length ) ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
  Cnt = 0 ;
  while ( Length > 0 )
  {
    WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) ( * Data ) ) ;
    Data ++ ;
    Length -- ;
    UART0SendString ( NumberStr ) ;
    Cnt ++ ;
    if ( Cnt == 14 )
    {
      UART0Send ( 0x0D ) ;
      UART0Send ( 0x0A ) ;
      while ( UART0Working ( ) )
      {
      } ;
      break;
    } /* if ... */
    else
    {
      UART0Send ( ' ' ) ;
    } /* if ... else */ ;
  } /* while */ ;
  Cnt = 0 ;
  while ( Length > 0 )
  {
    WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) ( * Data ) ) ;
    Data ++ ;
    Length -- ;
    UART0SendString ( NumberStr ) ;
    Cnt ++ ;
    if ( Cnt == 16 )
    {
      UART0Send ( 0x0D ) ;
      UART0Send ( 0x0A ) ;
      while ( UART0Working ( ) )
      {
      } ;
      Cnt = 0 ;
    } /* if ... */
    else
    {
      UART0Send ( ' ' ) ;
    } /* if ... else */ ;
  } /* while */ ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
} /* traceBlock */
