/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                       MODULE UDP Protocol Service                      **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#ifndef _udp_service_
#define _udp_service_

#include "types.h"
#include "ipservice.h"


#define UDPHeaderSize                 8

#define AutoIncrementUDPPortNoLoBound 2048
#define AutoIncrementUDPPortNoUpBound 4095

struct _UDPLayerInstanceRec ;
typedef struct _UDPLayerInstanceRec * PtrUDPLayerInstanceRecT ;

struct _UDPSocketDescriptionRec ;
typedef struct _UDPSocketDescriptionRec * PtrUDPSocketDescriptionRecT ;

typedef void ( * AnyUDPRecvEventServProcT ) ( ULONG  /* SenderIPAddress */ ,
                                              USHORT /* SenderPortNo    */ ,
                                              ULONG  /* DestIPAddress   */ ,
                                              USHORT /* DestPortNo      */ ,
                                              void * /* UDPDataBuffer   */ ,
                                              USHORT /* UDPDataSize     */ ) ;

typedef void ( * SocketUDPRecvEventServProcT ) ( PtrUDPSocketDescriptionRecT /* UDPSocket       */ ,
                                                 ULONG                       /* SenderIPAddress */ ,
                                                 USHORT                      /* SenderPortNo    */ ,
                                                 ULONG                       /* DestIPAddress   */ ,
                                                 USHORT                      /* DestPortNo      */ ,
                                                 void *                      /* UDPDataBuffer   */ ,
                                                 USHORT                      /* UDPDataSize     */ ) ;

typedef struct _UDPSocketDescriptionRec
               {
                 USHORT                       ListenPortNo ;
                 SocketUDPRecvEventServProcT  ListenService ;
                 PtrUDPLayerInstanceRecT      UDPLayerInstanceLink ;
                 PtrUDPSocketDescriptionRecT  NextSocketLink ;
               } UDPSocketDescriptionRecT ;

typedef struct _UDPLayerInstanceRec
               { PtrIPLayerInstanceRecT       IPLayerInstance ;
                 AnyUDPRecvEventServProcT     DefaultListenService ;
                 PtrUDPSocketDescriptionRecT  ListenSocketTopEntry ;
                 USHORT                       LastLocalPort ;
               } UDPLayerInstanceRecT ;

#define UDP_SendOK           0
#define UDP_SendFailed       1
#define UDP_SendSegmentToBig 2
#define UDP_SendRouteFailed  3

extern USHORT UDPSendData ( PtrUDPSocketDescriptionRecT /* Socket        */ ,
                            ULONG                       /* DestIPAddress */ ,
                            USHORT                      /* DestPortNo    */ ,
                            void *                      /* DataBufferPtr */ ,
                            USHORT                      /* DataSize      */ ) ;

extern PtrUDPSocketDescriptionRecT AllocateUDPSocket ( PtrUDPLayerInstanceRecT /* UDPLayerInstance */ ) ;

extern void DeallocateUDPSocket ( PtrUDPSocketDescriptionRecT /* UDPSocket */ ) ;

#define UDP_BindPortOK              0
#define UDP_BindPortDupl            1

extern USHORT BindUDPSocket ( PtrUDPSocketDescriptionRecT /* UDPSocket */ ,
                              USHORT                      /* PortNo    */ ) ;

extern void SetUDPListen ( PtrUDPSocketDescriptionRecT /* UDPSocket       */ ,
                           SocketUDPRecvEventServProcT /* UDPDataConsumer */ ) ;

extern void SetDefaultUDPListen ( PtrUDPLayerInstanceRecT  /* UDPLayerInstance       */ ,
                                  AnyUDPRecvEventServProcT /* DefaultUDPDataConsumer */ ) ;

extern PtrUDPSocketDescriptionRecT CreateUDPSocket ( PtrUDPLayerInstanceRecT /* UDPLayerInstance */ ) ;

extern void UDPInitInstance ( PtrUDPLayerInstanceRecT /* UDPLayerInstance */ ,
                              PtrIPLayerInstanceRecT  /* IPLayerInstance  */ ) ;

extern PtrUDPLayerInstanceRecT GetUDPInstance ( PtrUDPSocketDescriptionRecT /* UDPSocket */ ) ;

extern PtrIPLayerInstanceRecT GetIPInstanceFromUDPInstance ( PtrUDPLayerInstanceRecT /* UDPLayerInstance */ ) ;

extern PtrUDPLayerInstanceRecT StandardAddUDPInstance ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ) ;

#endif
